Attribute VB_Name = "FormPosition"
Option Explicit

Private Type POINTAPI
    X As Long
    Y As Long
End Type
Private Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type
Private Type WINDOWPLACEMENT
    Length As Long
    flags As Long
    showCmd As Long
    ptMinPosition As POINTAPI
    ptMaxPosition As POINTAPI
    rcNormalPosition As RECT
End Type

Private Declare Function GetWindowPlacement Lib "user32" (ByVal hWnd As Long, lpwndpl As WINDOWPLACEMENT) As Long
Private Declare Function SetWindowPlacement Lib "user32" (ByVal hWnd As Long, lpwndpl As WINDOWPLACEMENT) As Long

Private Const SW_RESTORE = 9
Private Const SW_MINIMIZE = 6
Private Const SW_NORMAL = 1
Private Const SW_SHOWMINIMIZED = 2

Private Const SECTION_NAME = "Geometry"

' Load the form's geometry from the registry.
Public Sub RestorePosition(ByVal frm As Form, ByVal app_name As String)
Dim wp As WINDOWPLACEMENT
Dim window_state As Long

    ' Initialize wp from the registry.
    With wp
        .Length = Len(wp)
        .flags = GetSetting(app_name, SECTION_NAME, "flags", 0)
        .showCmd = GetSetting(app_name, SECTION_NAME, "showCmd", SW_RESTORE)
        .ptMinPosition.X = GetSetting(app_name, SECTION_NAME, "MinX", 100)
        .ptMinPosition.Y = GetSetting(app_name, SECTION_NAME, "MinY", 100)
        .rcNormalPosition.Left = GetSetting(app_name, SECTION_NAME, "Left", 0)
        .rcNormalPosition.Top = GetSetting(app_name, SECTION_NAME, "Top", 0)
        .rcNormalPosition.Right = GetSetting(app_name, SECTION_NAME, "Right", 100)
        .rcNormalPosition.Bottom = GetSetting(app_name, SECTION_NAME, "Bottom", 100)
    End With

    ' Position the form.
    SetWindowPlacement frm.hWnd, wp
End Sub

' Save the form's geometry in the registry.
Public Sub SavePosition(ByVal frm As Form, ByVal app_name As String)
Dim disallow_minimized As Boolean
Dim wp As WINDOWPLACEMENT

    ' Do nothing if the form is not visible.
    If frm.Visible = False Then Exit Sub

    ' Comment this line if you want to
    ' allow the form to come up minimized.
    disallow_minimized = True

    ' Get the window's normalized placement.
    wp.Length = Len(wp)
    GetWindowPlacement frm.hWnd, wp

    With wp
        If disallow_minimized And _
            ((.showCmd = SW_MINIMIZE) Or _
             (.showCmd = SW_SHOWMINIMIZED)) _
        Then
            SaveSetting app_name, SECTION_NAME, "showCmd", SW_NORMAL
        Else
            SaveSetting app_name, SECTION_NAME, "showCmd", .showCmd
        End If

        SaveSetting app_name, SECTION_NAME, "flags", .flags
        SaveSetting app_name, SECTION_NAME, "MinX", .ptMinPosition.X
        SaveSetting app_name, SECTION_NAME, "MinY", .ptMinPosition.Y
        SaveSetting app_name, SECTION_NAME, "Left", .rcNormalPosition.Left
        SaveSetting app_name, SECTION_NAME, "Top", .rcNormalPosition.Top
        SaveSetting app_name, SECTION_NAME, "Right", .rcNormalPosition.Right
        SaveSetting app_name, SECTION_NAME, "Bottom", .rcNormalPosition.Bottom
    End With
End Sub


